from .archive_provider import ArchiveProvider
from .provider.tar_provider import TarProvider
from .provider.zip_provider import ZipProvider
from .provider.compressed_tar_provider import GZipProvider, BZip2Provider

name = 'slipit'
